process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0'
process.env.SUPPRESS_NO_CONFIG_WARNING = true

const fixtures = require('./../fixtures/fixtures')
const Model = require('./../helpers/Model')
const cloneDeep = require('lodash/cloneDeep')
const requests = require('./../helpers/requests')

const path = '../../routes/collectionsInvitations/get'
const route = {
  path: '/api/collections/:collectionId/invitations/:invitationId?',
}
describe('Get collection invitations route handler', () => {
  let testFixtures = {}
  let models
  beforeEach(() => {
    testFixtures = cloneDeep(fixtures)
    models = Model.build(testFixtures)
  })
  it('should return success when the request data is correct', async () => {
    const { editorInChief, handlingEditor } = testFixtures.users
    const { collection } = testFixtures.collections
    const res = await requests.sendRequest({
      userId: editorInChief.id,
      route,
      models,
      path,
      query: {
        role: 'handlingEditor',
        userId: handlingEditor.id,
      },
      params: {
        collectionId: collection.id,
      },
    })

    expect(res.statusCode).toBe(200)
    const data = JSON.parse(res._getData())
    expect(data.length).toBeGreaterThan(0)
  })
  it('should return an error when parameters are missing', async () => {
    const { editorInChief } = testFixtures.users
    const res = await requests.sendRequest({
      userId: editorInChief.id,
      route,
      models,
      path,
    })
    expect(res.statusCode).toBe(400)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('Role is required')
  })
  it('should return an error when the collection does not exist', async () => {
    const { editorInChief, handlingEditor } = testFixtures.users
    const res = await requests.sendRequest({
      userId: editorInChief.id,
      route,
      models,
      path,
      query: {
        role: 'handlingEditor',
        userId: handlingEditor.id,
      },
      params: {
        collectionId: 'invalid-id',
      },
    })
    expect(res.statusCode).toBe(404)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('collection not found')
  })
  it('should return an error when the role is invalid', async () => {
    const { editorInChief, handlingEditor } = testFixtures.users
    const { collection } = testFixtures.collections
    const res = await requests.sendRequest({
      userId: editorInChief.id,
      route,
      models,
      path,
      query: {
        role: 'invalidRole',
        userId: handlingEditor.id,
      },
      params: {
        collectionId: collection.id,
      },
    })
    expect(res.statusCode).toBe(400)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual(`Role invalidRole is invalid`)
  })
  it('should return success with an empty array when the collection does not have a the requested role team', async () => {
    const { editorInChief, handlingEditor } = testFixtures.users
    const { collection } = testFixtures.collections
    delete collection.invitations
    const res = await requests.sendRequest({
      userId: editorInChief.id,
      route,
      models,
      path,
      query: {
        role: 'author',
        userId: handlingEditor.id,
      },
      params: {
        collectionId: collection.id,
      },
    })
    expect(res.statusCode).toBe(200)
    const data = JSON.parse(res._getData())
    expect(data).toHaveLength(0)
  })
  it('should return an error when a user does not have invitation rights', async () => {
    const { author } = testFixtures.users
    const { collection } = testFixtures.collections
    const res = await requests.sendRequest({
      userId: author.id,
      route,
      models,
      path,
      query: {
        role: 'reviewer',
      },
      params: {
        collectionId: collection.id,
      },
    })
    expect(res.statusCode).toBe(403)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('Unauthorized.')
  })
})
